package Web;
use Moose;
use Data::Dumper;
use Plack::Request;
use base 'Plack::Component';
#use Date::Manip;
use DBI;
use JSON;
use Time::HiRes;
use Search::QueryParser;

has 'log' => ( is => 'ro', isa => 'Log::Log4perl::Logger', required => 1 );
has 'conf' => ( is => 'ro', isa => 'Config::JSON', required => 1 );
has 'db' => ( is => 'rw', isa => 'Object', required => 0 );
has 'parser' => ( is => 'rw', isa => 'Object', required => 0 );
has 'json' => ( is => 'rw', isa => 'Object', required => 0 );

sub BUILD {
	my ($self, $params) = @_;
	
	$self->db(DBI->connect($self->conf->get('db/dsn'), $self->conf->get('db/username'), $self->conf->get('db/password')));
	
	$self->parser(Search::QueryParser->new(rxField => qr/[\.\w]+/));
	$self->json(JSON->new->pretty);
		
	return $self;
}

sub call {
	my ($self, $env) = @_;
	
	my $req = Plack::Request->new($env);
	my $res = $req->new_response(200); # new Plack::Response
	$res->content_type('application/json');
	
	my $body;
	eval {
		if ($req->query_parameters->{q}){
			my $query = $self->parser->parse($req->query_parameters->{q});
			$self->log->debug('query_object: ' . Dumper($query));
			my $ret = $self->_query($query);
			$body = $self->json->encode({ aaData => $ret});
		}
		else {
			$body = $self->_index();
			$res->content_type('text/html');
		}
	};
	if ($@){
		my $e = $@;
		$self->log->error($e);
		$body = $e . "\n";
	}
    $res->body($body);
    $res->finalize;
}

sub _index {
	my $self = shift;
	my $body = <<EOT
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
	<head>
                <meta http-equiv="content-type" content="text/html; charset=utf-8" />
                <title>DataTables example</title>
<script type="text/javascript" language="javascript" src="inc/jquery.js"></script>
<script type="text/javascript" language="javascript" src="inc/jquery.dataTables.js"></script>
<script type="text/javascript" language="javascript" src="inc/app.js"></script>
	</head>
	<body>
		<form id="search">
			<input name="q" id="search_q" size="50" value="" />
			<input type="submit" id="search_submit" />
		</form>
		<div id="dt_app">
			<div id="container">
				<table cellpadding="0" cellspacing="0" border="0" class="display" id="app">
				</table>
			</div>
		</div>
	</body>
</html>
EOT
;
	return $body;
}

sub _query {
	my $self = shift;
	my $query_object = shift;
	
	my ($query, $sth);
	$query = 'SELECT data FROM data ';
	my $counter = 1;
	my $where_clause = 'WHERE 1=1 ';
	my ($limit, $offset) = (100, 0);
	my @placeholders;
	foreach my $symbol ('', '+', '-'){
		foreach my $hash (@{ $query_object->{$symbol} }){
			if (lc($hash->{field}) eq 'limit'){
				$limit = int($hash->{value});
				next;
			}
			if (lc($hash->{field}) eq 'offset'){
				$offset = int($hash->{value});
				next;
			}
			my $boolean = '';
			if ((($symbol eq '' or $symbol eq '+') and $hash->{op} eq '-') or
				($symbol eq '-'  and ($hash->{op} eq '=' or $hash->{op} eq ':'))){
				$boolean = 'NOT ';
			}
			my $op = ' = ';
			if ($hash->{value} =~ /([^\*]+)\*$/){
				$op = ' LIKE ';
				$hash->{value} = $1 . '%';
			}
			
			$query .= 'JOIN kv t' . $counter . ' ON (data.id=t' . $counter . '.data_id) ';
			if ($hash->{field}){
				$where_clause .= ' AND t' . $counter .'.k=?';
				push @placeholders, $hash->{field};
			}
			$where_clause .= ' AND ' . $boolean . 't' . $counter . '.v' . $op .'?';
			push @placeholders, $hash->{value};
			$counter++;
		}
	}
	push @placeholders, $offset, $limit;
	
	die('No query terms found') unless $counter > 1;
	$query .= $where_clause . ' LIMIT ?,?';
	$self->log->debug('query: ' . $query . ', placeholders: ' . join(',', @placeholders));
	$sth = $self->db->prepare($query);
	$sth->execute(@placeholders);
	my @ret;
	while (my $row = $sth->fetchrow_hashref){
		my $data = $self->json->decode($row->{data});
		foreach my $key (keys %$data){
			my $val = delete $data->{$key};
			$key =~ s/\./\_/g; #must strip any dots
			$data->{$key} = $val;
		}
		push @ret, $data;
	}
	return \@ret;
}

1;